@@ -2,7 +2,8 @@
 
 use deno_core::error::AnyError;
 use deno_core::FsModuleLoader;
-use deno_runtime::deno_file::BlobUrlStore;
+use deno_runtime::deno_broadcast_channel::InMemoryBroadcastChannel;
+use deno_runtime::deno_web::BlobUrlStore;
 use deno_runtime::permissions::Permissions;
 use deno_runtime::worker::MainWorker;
 use deno_runtime::worker::WorkerOptions;
@@ -40,8 +41,9 @@ async fn main() -> Result<(), AnyError> {
     no_color: false,
     get_error_class_fn: Some(&get_error_class_name),
     location: None,
-    location_data_dir: None,
+    origin_storage_dir: None,
     blob_url_store: BlobUrlStore::default(),
+    broadcast_channel: InMemoryBroadcastChannel::default(),
   };
 
   let js_path =
@@ -53,6 +55,6 @@ async fn main() -> Result<(), AnyError> {
     MainWorker::from_options(main_module.clone(), permissions, &options);
   worker.bootstrap(&options);
   worker.execute_module(&main_module).await?;
-  worker.run_event_loop().await?;
+  worker.run_event_loop(false).await?;
   Ok(())
 }
