@@ -38,8 +38,6 @@
     return core.opAsync("op_host_get_message", id);
   }
 
-  const decoder = new TextDecoder();
-
   /**
    * @param {string} permission
    * @return {boolean}
@@ -166,7 +164,7 @@
 
       this.#name = name;
       const hasSourceCode = false;
-      const sourceCode = decoder.decode(new Uint8Array());
+      const sourceCode = core.decode(new Uint8Array());
 
       if (
         specifier.startsWith("./") || specifier.startsWith("../") ||
@@ -192,16 +190,16 @@
       this.#poll();
     }
 
-    #handleMessage = (data) => {
+    #handleMessage(data) {
       const msgEvent = new MessageEvent("message", {
         cancelable: false,
         data,
       });
 
       this.dispatchEvent(msgEvent);
-    };
+    }
 
-    #handleError = (e) => {
+    #handleError(e) {
       const event = new ErrorEvent("error", {
         cancelable: true,
         message: e.message,
@@ -219,7 +217,7 @@
       }
 
       return handled;
-    };
+    }
 
     #poll = async () => {
       while (!this.#terminated) {
