@@ -1,9 +1,9 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 
+pub use deno_broadcast_channel;
 pub use deno_console;
 pub use deno_crypto;
 pub use deno_fetch;
-pub use deno_file;
 pub use deno_timers;
 pub use deno_url;
 pub use deno_web;
@@ -15,7 +15,7 @@ pub use deno_webstorage;
 pub mod colors;
 pub mod errors;
 pub mod fs_util;
-pub mod inspector;
+pub mod inspector_server;
 pub mod js;
 pub mod metrics;
 pub mod ops;
